SystemOrganization addCategory: #'Sake-Packages'!

!MCWorkingCopy methodsFor: '*sake-packages' stamp: 'test 12/23/2008 02:53'!
myMenuForPackages: aMenu

	(self packageInfo propertyAt: #packages) ifNotNilDo: [ :pkgName |
		aMenu
			add: 'load latest release using Sake/Packages' 
			target: (Packages current named: pkgName) 
			selector: #run.
		
		aMenu 
			add: 'load latest code using Sake/Packages' 
			target: (Packages beta named: pkgName) 
			selector: #run.		
	]. ! !

SakeTask subclass: #Packages
	instanceVariableNames: 'unloadBlock unloadPriors provides name version isUnload'
	classVariableNames: 'Provided'
	poolDictionaries: ''
	category: 'Sake-Packages'!

Packages class
	instanceVariableNames: 'lastUpdate theUUniverse'!

!Packages commentStamp: 'test 12/23/2008 06:17' prior: 0!
To generate all of the methods based upon universes definitions:
	
 	Packages taskGenerateAllUniverses run.
	or
	Packages taskGenerateAll  run.

Sake/Packages usage:

Public API
============
Package definition for your current version of Squeak are found using the following path.

For SystemVersion Squeak3.7

Packages findPath  -> {PackagesSqueak37 . PackagesDev . PackagesBeta} *the default is #current
Packages current findPath  -> {PackagesSqueak37 . PackagesDev . PackagesBeta}
Packages dev findPath  {PackagesDev. PackagesSqueak37 . PackagesBeta}
Packages beta findPath {PackagesBeta . PackagesSqueak37 . PackagesDev}

Packages current load: 'Seaside'.
Packages beta named: 'Seaside'.

or

(Packages current named: 'Seaside') run.  " or runQuietly, runStepping, runLogging"
(Packages beta named: 'Seaside') run.

multiples:

Packages current load: #('Seaside' 'Magma' 'Logging')  

Run-variants
=========

#runStepping , - presents a confirm/debug dialog before each action.
#run                 - default.
#runQuietly     - auto-confirms any SakeConfirm dialogs.
#runLogging    - Writes any SakeStop warnings to self log.

Unloading
========
Unloading comes in two variants.

Each package task loaded by Sake/Packages is remembered in the 'provided' list
If you perform:

    Packages unload: 'Seaside' .

    Packages unloadStepping: 'Seaside' .

Then the 'historical' unload scripts are used, as defined when the original load tasks were run.

If instead you perform:

    (Packages current named: 'Seaside') unload runStepping.
 
Then the most recently defined unload script will be run.

Note: If packages such as "Magma server" and "Magma client" provides "Magma", then

    Packages unload: 'Magma'.

Will unload whichever of the two are loaded.
===
Misc notes...

Universes are using 'instance side' task definition, so the task extensions mechanism does not work in this context.

If a package appears in Packages under an obscure name, it can
tell the PackageInfo instance what name was used to load it via metadata
at mcName:

If a package has a version number with a '+' after it, then 'Packages upgrade' will always attempt to load the latest code, leaving Monticello to determing if there are any code changed that need to be applied.
	!

Packages class
	instanceVariableNames: 'lastUpdate theUUniverse'!

!Packages class methodsFor: 'public' stamp: 'kph 4/20/2008 04:22'!
UUniverse

	^ Smalltalk classNamed: 'UUniverse'
! !

!Packages class methodsFor: 'public' stamp: 'damiencassou 12/23/2008 11:21'!
allPackages
	^ (self allSelectorsBelow: Packages) collect: [:selector | self named: selector]! !

!Packages class methodsFor: 'public' stamp: 'damiencassou 12/23/2008 11:51'!
allPackagesDo: aBlock

	^ (self allSelectorsBelow: Packages) do: [ :ea | aBlock value: (self named: ea) ]! !

!Packages class methodsFor: 'utils' stamp: 'kph 5/26/2008 12:02'!
asSelector: aName
	| toUse |
 
	toUse := ''.
	aName do:
		[:char | char isAlphaNumeric ifTrue: [toUse := toUse copyWith: char]].
	(aName size == 0 or: [aName first isLetter not])
		ifTrue:		[toUse := 'v', toUse].

	^ toUse asSymbol 
 ! !

!Packages class methodsFor: 'private' stamp: 'kph 4/10/2008 08:11'!
basicNamed: packageName
	
	^ self basicNew perform: (self asSelector: packageName); initialize; yourself! !

!Packages class methodsFor: 'public' stamp: 'kph 9/4/2008 14:16'!
beta

	^ PackagesBeta! !

!Packages class methodsFor: 'public' stamp: 'kph 9/4/2008 14:28'!
current

	^ (Smalltalk classNamed: ('Packages' , SystemVersion current majorMinorVersion asLegalSelector capitalized) asSymbol) ifNil: [ Packages ]
! !

!Packages class methodsFor: 'public' stamp: 'kph 9/4/2008 14:25'!
dev

	^ PackagesDev! !

!Packages class methodsFor: 'private' stamp: 'kph 9/4/2008 15:18'!
findNamed: packageName 
	 
	| sel |
	
	sel := self asSelector: packageName.
	
	^ (self findPath detect: [ :ea | ea canPerform: sel ] ifNone: [ self error: 'not found' ]) basicNamed: packageName! !

!Packages class methodsFor: 'private' stamp: 'kph 12/1/2008 19:18'!
findPath

	^ { self current. self dev. self beta. }! !

!Packages class methodsFor: 'as yet unclassified' stamp: 'kph 1/27/2008 22:56'!
freeSomeSpace

	self initialize! !

!Packages class methodsFor: 'initialization' stamp: 'kph 9/4/2008 17:25'!
initialize

	lastUpdate := nil.! !

!Packages class methodsFor: 'accessors' stamp: 'kph 12/11/2008 13:51'!
isPackageProvided: aPackage

	| pkgInfo |
	pkgInfo := PackageOrganizer packageNamed: aPackage name ifAbsent: [ nil ].! !

!Packages class methodsFor: 'testing' stamp: 'kph 1/27/2008 20:30'!
isUniverse

	^ self class includesSelector: #universeSelector
	 ! !

!Packages class methodsFor: 'accessors' stamp: 'kph 12/28/2007 19:32'!
lastUpdate

	^ lastUpdate ifNil: [ lastUpdate := DateAndTime now ] ! !

!Packages class methodsFor: 'launcher' stamp: 'kph 7/7/2008 15:13'!
launchFrom: launcher

	^self launchWith: launcher getParameters! !

!Packages class methodsFor: 'launcher' stamp: 'kph 11/2/2008 01:03'!
launchWith: params

	| packagesClass task | 
	
	packagesClass := self current.
	
 	params at: 'B' ifPresent: [ :v | params at: 'BETA' put: v ].
 	params at: 'U' ifPresent: [ :v | params at: 'UNLOAD' put: v ].
 	params at: 'L' ifPresent: [ :v | params at: 'LOAD' put: v ].
 
	params at: 'BETA' ifPresent: [ :v | 
		v ifTrue: [ packagesClass := self beta ].
	].

	params at: 'LOAD' ifPresent: [ :v | 
		task := packagesClass collection: (v findTokens: ';')		
	].
 
	params at: 'UNLOAD' ifPresent: [ :v | 
		task := SakeTask collection: ((v findTokens: ';') collect: [ :ea | (self named: ea) unload ])	
	].

 
	task launch: params.
	
	^ true! !

!Packages class methodsFor: 'public' stamp: 'kph 5/16/2008 01:18'!
load: aNameOrList

	^ (self named: aNameOrList) run! !

!Packages class methodsFor: 'public' stamp: 'kph 9/4/2008 17:05'!
named: packageNameOrList
	
	packageNameOrList isString ifTrue: [ ^ self findNamed: packageNameOrList ].
	
	^ (packageNameOrList collect: [ :ea | self findNamed: packageNameOrList ]) asTask! !

!Packages class methodsFor: 'accessors' stamp: 'kph 1/28/2008 12:06'!
provided

	^ Provided ifNil: [ Provided := Dictionary new ]! !

!Packages class methodsFor: 'public' stamp: 'kph 4/10/2008 07:45'!
providedAt: aName

	^ self provided at: aName! !

!Packages class methodsFor: 'public' stamp: 'kph 4/21/2008 01:27'!
squeakmap

	^ Smalltalk classNamed: #PackagesSqueakMap! !

!Packages class methodsFor: 'public' stamp: 'kph 6/3/2008 01:11'!
string: aPackageName

	^ self named: aPackageName! !

!Packages class methodsFor: 'tasks - universes' stamp: 'kph 4/24/2008 20:23'!
taskGenerateAll
 
^ SakeTask define: [ :task | 
	 
	task dependsOn: { 
	
	self taskGenerateAllUniverses 
	
	 }.
		
	task if: [ self squeakmap notNil ].
	
	task action: { self squeakmap taskGenerateSqueakMapPackageTasks }

]
! !

!Packages class methodsFor: 'tasks - universes' stamp: 'kph 9/4/2008 17:25'!
taskGenerateAllUniverses
 
"
self taskGenerateAllUniverses run.
"

^ SakeTask define: [ :task | 

	task if: { (SakeTask class: 'UUniverse') exists }.
	
	task action: 
	
		(Packages allSubclasses select: [ :each | each initialize isUniverse ] thenCollect: [ :sc | sc taskGenerateUniversePackageTasks ]). 
	 
]
! !

!Packages class methodsFor: 'accessors' stamp: 'test 12/22/2008 17:00'!
taskGenerateFiles

	^ lastUpdate ifNil: [ lastUpdate := DateAndTime now ] ! !

!Packages class methodsFor: 'tasks - universes' stamp: 'kph 11/12/2008 16:57'!
taskGenerateUniversePackageTasks

| source selector |
^ SakeTask define: [ :task | 
	
	task dependsOn: { 
	
		[ self isUniverse ]. 
 		self taskUpdateUniverse.
		self taskRemoveOldPackages.
	}.

	task action: [ 
		(self theUUniverse packageNames collect: [ :each | self theUUniverse newestPackageNamed: each ]) do: [ :each | 
			source := (WriteStream on: String new).
				source << (selector := self asSelector: each name).
				source cr; cr.
				source << '	self name: ' << each name printString << '.' ; cr.
				source << '	self version: ''' << each version << '''.' ; cr.
				 					
				source << '	info category: ''' << each category printString << '''.' ; cr.
				source << '	info description: ' ; cr.
				source << each description withSqueakLineEndings printString << '.' ; cr.
				source << '	info maintainer: ''' << each maintainer << '''.' ; cr.
				source << '	info homepage: ''' << (each homepage ifNil: ['']) asString << '''.' ; cr.					
				source << '	info squeakMapID: ''' << (each squeakMapID ifNil: [''])  asString << '''.' ; cr.					
				source << '	info url: ' << (each url ifNotNil: [  '''' , each url printString , '''' ] ifNil: [ 'nil' ]) << '.' ; cr.					
				source << '	self provides: ' << each provides asArray printString << '.' ; cr ; cr.				
				source << '	self dependsOn: ' << each depends asSortedCollection asArray printString << '.' ; cr ; cr.
 
			(self sourceCodeAt: selector ifAbsent: nil)	 ~= source contents ifTrue: [								 
			 
				self compile: source contents classified: each category printString notifying: nil
			]
		].
		theUUniverse := nil.
	].	
]! !

!Packages class methodsFor: 'tasks - universes' stamp: 'kph 5/26/2008 12:03'!
taskRemoveOldPackages
 
^ SakeTask define: [ :task | 

	task dependsOn: { [ self isUniverse ]. self taskUpdateUniverse. }.
		
	task action: [ 
		(self selectors difference: 
			(self theUUniverse packageNames collect: [ :each | self asSelector: each ]))
			do: [ :old | self removeSelector: old ]
	].	
  ]! !

!Packages class methodsFor: 'tasks - universes' stamp: 'kph 4/22/2008 21:35'!
taskUpdateUniverse

^ SakeTask define: [ :task | 
	
	task dependsOn: { [ self isUniverse ] }.

	task if: [ theUUniverse isNil or: [ DateAndTime now - self lastUpdate > 10 minutes ] ].

	task action: [ 
		self theUUniverse updatePackagesViaWWW. 
		lastUpdate := DateAndTime now 
	].	

]! !

!Packages class methodsFor: 'tasks - universes' stamp: 'kph 9/4/2008 17:27'!
taskValidate 
 
"
self taskValidateAll run.
"

^ SakeTask define: [ :task | 
	
	task action:  { 	 
					self taskValidateCategories }
	 
]
! !

!Packages class methodsFor: 'tasks - universes' stamp: 'kph 9/4/2008 17:27'!
taskValidateAll
 
"
self taskValidateAll run.
"

^ SakeTask define: [ :task | 
	
	task action: 
	
		(Packages allSubclasses select: [ :each | each initialize isUniverse ] thenCollect: [ :sc | sc taskValidate ]). 
	 
]
! !

!Packages class methodsFor: 'tasks - universes' stamp: 'kph 8/12/2008 22:03'!
taskValidateCategories
 
^ SakeTask define: [ :task | 
	
	task action: [
		self selectors do: [ :sel | 
			self organization classify: sel under: (((self named: sel) info category) ifNil: [ 'as yet unclassified' ])
		].
		self organization removeEmptyCategories. 
	]
]


! !

!Packages class methodsFor: 'accessors' stamp: 'kph 4/20/2008 16:43'!
theUUniverse

	^ theUUniverse 
	ifNil: [ theUUniverse := (Smalltalk classNamed: 'UUniverse') perform: self universeSelector asSymbol ]! !

!Packages class methodsFor: 'public' stamp: 'kph 4/20/2008 16:38'!
universe

	^ Smalltalk classNamed: (('Packages' , SystemVersion current majorMinorVersion asLegalSelector capitalized), 'U') asSymbol  

! !

!Packages class methodsFor: 'public' stamp: 'kph 5/16/2008 01:25'!
unload: aNameOrList

	aNameOrList isString ifTrue: [ ^ (self providedAt: aNameOrList) unload run].
	
	^ (aNameOrList collect: [ :ea |  (self providedAt: aNameOrList) unload ]) asTask run
 ! !

!Packages class methodsFor: 'public' stamp: 'kph 5/16/2008 01:25'!
unloadStepping: aNameOrList

	aNameOrList isString ifTrue: [ ^ (self providedAt: aNameOrList) unload runStepping].
	
	^ (aNameOrList collect: [ :ea |  (self providedAt: aNameOrList) unload ]) asTask runStepping
 ! !

!Packages class methodsFor: 'public' stamp: 'test 12/23/2008 06:29'!
update
	"load the latest definitions"
	Packages current load: 'Packages'.
! !

!Packages class methodsFor: 'public' stamp: 'test 12/23/2008 06:31'!
upgrade
	Packages update.
	
	self provided values asSet asTask run! !

!Packages class methodsFor: 'public' stamp: 'test 12/23/2008 06:31'!
upgradeList
	"the list of pacakges that would be loaded by an upgrade"
	Packages beta load: 'Packages'.
	^ (self provided values asSet asTask instVarNamed: 'actionBlock') select: [ :ea | ea isNeeded: #() ] thenCollect: [ :ea | ea name ].
	
	"
	self upgradeList
	"! !

!Packages methodsFor: 'comparing' stamp: 'kph 4/24/2008 17:41'!
= other 

^ (super = other) and: [ self version = other version ]
! !

!Packages methodsFor: 'accessing' stamp: 'kph 8/19/2008 00:10'!
action

	^ self isUnloading 
		ifTrue: [ unloadBlock ifNil: [ [ self defaultUnloadAction ] ] ]
		ifFalse: [ super action ].! !

!Packages methodsFor: 'as yet unclassified' stamp: 'test 12/23/2008 03:08'!
addToPackageInfo

    "
    Here we tell the PackageInfo instance what name was used to load it.

   if a package appears in PackageInfo under a different name to which it
    appears in Packages, the PackageInfo name can be told to us via
    metadata #mcName:  Example: 'Seaside' is in PackageInfo as 'Seaside2'.   
    "
	
	(self info mcName ifNil: [ self name ]) ifNotNilDo: [ :mcName |
		(PackageOrganizer default packageNamed: mcName ifAbsent: [ ^ self ])
			in: [ :pkgInfo |
				(pkgInfo respondsTo: #propertyAt:put:)
					ifTrue: [ pkgInfo propertyAt: #packages put: self name ].
			].
	]. 
	! !

!Packages methodsFor: 'Packages-Core' stamp: 'kph 8/19/2008 18:50'!
allDependants

	| list |

	list := OrderedCollection new.

	self provides in: [ :tgt |

		self class allPackagesDo: [ :package |

			package withAllPriorTasksIgnoringErrorsDo: [ :ea |
				(ea dependsOn includesAnyOf: tgt) ifTrue: [ (list includes: package) ifFalse: [ list add: package ] ].
			] 	
		]
	].

	^ list
	
"
String streamContents: [ :str | (Packages current named:'Seaside') allDependants do: [ :ea | str << ea info name << String cr << ea info description << String cr << '======='; cr ] ]
"! !

!Packages methodsFor: 'as yet unclassified' stamp: 'kph 8/18/2008 01:07'!
beUnloading

	isUnload := true.
	hasRun := false.! !

!Packages methodsFor: 'defaults' stamp: 'kph 11/12/2008 15:39'!
defaultAction

	^ [ self info url ifNotNilDo: [ :url | Installer installUrl: url ] ]! !

!Packages methodsFor: 'defaults' stamp: 'kph 12/11/2008 23:29'!
defaultUnloadAction

	self info url ifNotNil: [ Installer mc unload: (self info mcName ifNil: [ self name]) ]! !

!Packages methodsFor: 'as yet unclassified' stamp: 'kph 4/10/2008 07:37'!
dependants

	^ self class provided select: [ :dependant  | 
		 dependant dependsOn includesAnyOf: self provides.
	].
! !

!Packages methodsFor: 'as yet unclassified' stamp: 'kph 8/18/2008 01:14'!
dependsOn

	self isUnloading ifFalse: [ ^ super dependsOn ].
	
	unloadPriors ifNil: [ self unloadDependsOn: self loadedDependantsUnloadTasks ].
	
	^ unloadPriors! !

!Packages methodsFor: 'as yet unclassified' stamp: 'test 12/23/2008 02:37'!
doActionEnd

	self isUnloading ifTrue: [ ^ self ].
	
	self class provided in: [ :reg |
		self provides do: [ :each | reg at: each put: self ].
	].

	"if a package appears in Packages under an obscure name, it can
	tell the PackageInfo instance what name was used to load it via metadata
	at mcName"
	
	self addToPackageInfo.
	! !

!Packages methodsFor: 'as yet unclassified' stamp: 'kph 9/2/2008 02:36'!
doActionStep

	name ifNil: [ ^ self ].
		
	 self isUnloading 
		ifTrue: [ self step: 'Confirm unload: ', self name printString ]
		ifFalse: [ self step: 'Confirm load: ', self name printString ]! !

!Packages methodsFor: 'comparing' stamp: 'damiencassou 12/11/2008 14:33'!
hash
	^ super hash bitXor: self version hash! !

!Packages methodsFor: 'comparing' stamp: 'kph 12/11/2008 14:17'!
isAlreadyProvided

	(self version includes: '+') ifTrue: [ ^ false ]. "means the package is being loaded without specifying any fixed version"

	^ (self class provided at: self name ifAbsent: [ ^ false ]) version = self version

! !

!Packages methodsFor: 'testing' stamp: 'kph 12/11/2008 15:26'!
isNeeded: priorTasks

	self isAlreadyProvided ifTrue: [ ^ false ].

	^ super isNeeded: priorTasks! !

!Packages methodsFor: 'testing' stamp: 'kph 8/18/2008 00:56'!
isUnloading

	^ isUnload = true! !

!Packages methodsFor: 'initialize' stamp: 'kph 4/9/2008 06:10'!
load: aBlock

	self action: aBlock! !

!Packages methodsFor: 'as yet unclassified' stamp: 'kph 6/3/2008 18:10'!
loadedDependants

	^ self class provided select: [ :dependant  | 
		 dependant dependsOn includesAnyOf: self provides.
	].
! !

!Packages methodsFor: 'as yet unclassified' stamp: 'kph 8/18/2008 01:07'!
loadedDependantsUnloadTasks

	"if the task is the provided one, then we return the provided dependants."
	
	self loadedDependants in: [ :deps |
	
		deps ifEmpty: [ ^ #() ].
	
	 	^ (self class provided at: self name ifAbsent: nil) == self
			ifTrue: [ (deps collect: [ :ea | ea copy beUnloading ]) ]
			ifFalse: [ (deps collect: [ : ea | (self class named: ea name) beUnloading ]) ]
	].! !

!Packages methodsFor: 'accessing' stamp: 'damiencassou 12/23/2008 14:00'!
longDescription
	^ self info printString! !

!Packages methodsFor: 'accessing' stamp: 'kph 9/2/2008 02:35'!
name

	^ name ifNil: [ 'No Name' ]! !

!Packages methodsFor: 'accessing' stamp: 'kph 4/24/2008 20:15'!
name: aPackageName

	"the method which names us is our context"
	self setContext: thisContext sender.
	name := aPackageName.
	self info name: aPackageName.! !

!Packages methodsFor: 'accessing' stamp: 'kph 4/21/2008 01:34'!
provides

	provides ifNil: [ provides := #() ].	
		
	(provides includes: self name) ifFalse: [ provides := provides copyWith: self name ].

	^ provides! !

!Packages methodsFor: 'accessing' stamp: 'kph 4/10/2008 07:36'!
provides: list

	provides := list.
	(list includes: self name) ifFalse: [ provides := provides copyWith: self name ].
! !

!Packages methodsFor: 'accessing' stamp: 'kph 4/10/2008 08:16'!
unload

	^ self copy beUnloading ! !

!Packages methodsFor: 'accessing' stamp: 'kph 4/9/2008 05:33'!
unload: aBlock

	unloadBlock := aBlock! !

!Packages methodsFor: 'as yet unclassified' stamp: 'kph 4/10/2008 09:12'!
unloadDependsOn: aList
	unloadPriors := aList reject: [ :ea | ea isNil ]
	
! !

!Packages methodsFor: 'accessing' stamp: 'kph 7/7/2008 19:59'!
version
	^ version! !

!Packages methodsFor: 'accessing' stamp: 'kph 7/7/2008 16:19'!
version: aPackageVersion

	"the method which versions us is our context"
	self setContext: thisContext sender.
 
	self info version: aPackageVersion.
	version := aPackageVersion.! !
Packages initialize!
